"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
function useLocalStorage(key, initialValue) {
    const [storedValue, setStoredValue] = (0, react_1.useState)(() => {
        const item = localStorage.getItem(key);
        try {
            return item ? JSON.parse(item) : initialValue;
        }
        catch (error) {
            return initialValue;
        }
    });
    const setValue = (value) => {
        try {
            const valueToStore = value instanceof Function ? value(storedValue) : value;
            setStoredValue(valueToStore);
            localStorage.setItem(key, JSON.stringify(valueToStore));
        }
        catch (error) {
            console.log(`Failed to store value '${value}' for key '${key}'`);
        }
    };
    (0, react_1.useEffect)(() => {
        const storageEventHandler = (event) => {
            var _a;
            if (event.storageArea === localStorage && event.key === key) {
                setStoredValue(JSON.parse((_a = event.newValue) !== null && _a !== void 0 ? _a : ''));
            }
        };
        window.addEventListener('storage', storageEventHandler, false);
        return () => window.removeEventListener('storage', storageEventHandler, false);
    });
    return [storedValue, setValue];
}
exports.default = useLocalStorage;
